package com.recovery.admin.boot.config.ds;

import com.alibaba.druid.pool.DruidDataSource;

import com.recovery.common.base.ds.DynamicDataSource;
import lombok.extern.slf4j.Slf4j;
import org.apache.ibatis.session.SqlSessionFactory;
import org.mybatis.spring.SqlSessionFactoryBean;
import org.mybatis.spring.SqlSessionTemplate;
import org.mybatis.spring.annotation.MapperScan;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.jdbc.datasource.DataSourceTransactionManager;

import javax.sql.DataSource;
import java.util.HashMap;
import java.util.Map;

/**
 * Created by YHYR on 2017-12-25
 */
@Slf4j
@Configuration
@MapperScan(basePackages = DataSourceConfig.PACKAGE, sqlSessionFactoryRef = "commonSqlSessionFactory")
public class DataSourceConfig {

    // 精确到模块目录，以便跟其他数据源隔离
    static final String PACKAGE = "com.recovery.admin.boot.mapper";
    static final String MAPPER_LOCATION = "classpath:mapper/*.xml";

    @Value("${master.datasource.url}")
    private String masterDBUrl;

    @Value("${master.datasource.username}")
    private String masterDBUser;

    @Value("${master.datasource.password}")
    private String masterDBPassword;

    @Value("${master.datasource.driverClassName}")
    private String masterDBDreiverName;


    @Bean(name = "commonDataSource")
    @Primary
    public DynamicDataSource dynamicDataSource(){
        DynamicDataSource dynamicDataSource = DynamicDataSource.getInstance();

        DruidDataSource masterDataSource = new DruidDataSource();
        try {
            masterDataSource.setUrl(masterDBUrl);
            masterDataSource.setUsername(masterDBUser);
            masterDataSource.setPassword(masterDBPassword);
            masterDataSource.setDriverClassName(masterDBDreiverName);
        }catch (Exception e){
            log.error(e.getMessage());
        }

        Map<Object,Object> map = new HashMap<>();
        map.put("master", masterDataSource);
        dynamicDataSource.setTargetDataSources(map);

        return dynamicDataSource;
    }

    @Bean(name = "commonSqlSessionFactory")
    @Primary
    public SqlSessionFactory sqlSessionFactory(
            @Qualifier("commonDataSource") DataSource dynamicDataSource)
            throws Exception {
        SqlSessionFactoryBean bean = new SqlSessionFactoryBean();
        bean.setDataSource(dynamicDataSource);
        bean.setMapperLocations(new PathMatchingResourcePatternResolver()
                .getResources(DataSourceConfig.MAPPER_LOCATION));
        return bean.getObject();

    }

    @Bean(name = "commonTransactionManager")
    @Primary
    public DataSourceTransactionManager commonTransactionManager() {
        return new DataSourceTransactionManager(dynamicDataSource());
    }


    @Bean(name = "commonSqlSessionTemplate")
    public SqlSessionTemplate sqlSessionTemplate(
            @Qualifier("commonSqlSessionFactory") SqlSessionFactory sqlSessionFactory)
            throws Exception {
        return new SqlSessionTemplate(sqlSessionFactory);
    }
}
