package com.recovery.admin.boot.interceptor;


import com.alibaba.fastjson.JSONObject;
import com.auth0.jwt.interfaces.DecodedJWT;

import com.recovery.admin.boot.exception.BusinessException;
import com.recovery.admin.boot.filter.RequestWrapper;
import com.recovery.common.base.constant.Constants;
import com.recovery.common.base.result.ResultCode;
import com.recovery.common.base.utils.JwtUtils;
import lombok.extern.slf4j.Slf4j;

import org.springframework.util.StringUtils;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Date;
import java.util.HashMap;

@Slf4j
public class JwtInterceptor implements HandlerInterceptor {
    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        HashMap<String, String> map=new HashMap<>();
        //获取请求参数
        RequestWrapper requestWrapper = new RequestWrapper(request);
        //这里getBodyString()方法无参数
        log.info("RequestBody: {}", requestWrapper.getBodyString());
        //从http请求头获取token

        String token = requestWrapper.getHeader(Constants.LOGIN_USRE_TOKEN);
        if (StringUtils.isEmpty(token)) {
            throw new BusinessException(ResultCode.TOKEN_INVALID_OR_EXPIRED);
        }
        //把变量放在request请求域中，仅可以被这次请求，即同一个requerst使用
        request.setAttribute(Constants.LOGIN_USRE_TOKEN,token);
        try {
            //如果验证成功放行请求
            DecodedJWT verify = JwtUtils.verifyToken(token);
            return true;
        }
        catch (Exception exception)
        {
            throw new BusinessException(ResultCode.TOKEN_INVALID_OR_EXPIRED);
        }
    }
    /**
     * @date: 2023/2/6 12:46
     * @author: zhouzhaodong
     * @description: 访问控制器方法后执行
     */
    @Override
    public void postHandle(HttpServletRequest request, HttpServletResponse response, Object handler, ModelAndView modelAndView) {
        log.info(new Date() + "--postHandle:" + request.getRequestURL());
    }

    /**
     * @date: 2023/2/6 12:46
     * @author: zhouzhaodong
     * @description: postHandle方法执行完成后执行，一般用于释放资源
     */
    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler, Exception ex) {
        log.info(new Date() + "--afterCompletion:" + request.getRequestURL());
    }

    public void renderJson(HttpServletResponse response, Object object) {
        response.reset();
        response.setHeader("Access-Control-Allow-Origin", "*");
        response.setHeader("Access-Control-Allow-Methods", "*");
        response.setHeader("Access-Control-Allow-Headers", "x-requested-with,content-type");
        response.setContentType("application/json");
        response.setCharacterEncoding("utf-8");
        try {
            response.getWriter().print(JSONObject.toJSONString(object));
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}

