package com.recovery.auth.config.ds;

import com.alibaba.druid.pool.DruidDataSource;
import org.apache.ibatis.session.SqlSessionFactory;
import org.mybatis.spring.SqlSessionFactoryBean;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.jdbc.datasource.DataSourceTransactionManager;

import javax.sql.DataSource;

//@Configuration
//// 扫描 Mapper 接口并容器管理
//@MapperScan(basePackages = TeachingDataSourceConfig.PACKAGE, sqlSessionFactoryRef = "teachingSqlSessionFactory")
public class TeachingDataSourceConfig {
    // 精确到 auth 目录，以便跟其他数据源隔离
    static final String PACKAGE = "com.recovery.auth.dao.mapper";
    static final String MAPPER_LOCATION = "classpath:mapper/*.xml";

    @Value("${admin.datasource.url}")
    private String url;

    @Value("${admin.datasource.username}")
    private String user;

    @Value("${admin.datasource.password}")
    private String password;

    @Value("${admin.datasource.driverClassName}")
    private String driverClass;

    @Bean(name = "adminDataSource")
    public DataSource cpdDataSource() {
        DruidDataSource dataSource = new DruidDataSource();
        dataSource.setDriverClassName(driverClass);
        dataSource.setUrl(url);
        dataSource.setUsername(user);
        dataSource.setPassword(password);
        return dataSource;
    }

    @Bean(name = "adminTransactionManager")
    public DataSourceTransactionManager courseTransactionManager() {
        return new DataSourceTransactionManager(cpdDataSource());
    }

    @Bean(name = "adminSqlSessionFactory")
    public SqlSessionFactory cpdSqlSessionFactory(@Qualifier("adminDataSource") DataSource cpdDataSource)
            throws Exception {
        final SqlSessionFactoryBean sessionFactory = new SqlSessionFactoryBean();
        sessionFactory.setDataSource(cpdDataSource);
        sessionFactory.setMapperLocations(new PathMatchingResourcePatternResolver()
                .getResources(TeachingDataSourceConfig.MAPPER_LOCATION));
        return sessionFactory.getObject();
    }
}
