package com.recovery.auth.service.impl;


import com.alibaba.fastjson.JSON;
import com.recovery.auth.exception.BusinessException;
import com.recovery.auth.feign.UserFeignClient;
import com.recovery.auth.security.details.user.JwtAuthenticationRequest;
import com.recovery.auth.security.details.user.SysUserDetails;
import com.recovery.auth.service.AuthService;
import com.recovery.common.base.dto.UserAuthDTO;
import com.recovery.common.base.dto.UserAuthorityDto;
import com.recovery.common.base.result.ApiResult;
import com.recovery.common.base.result.ResultCode;
import com.recovery.common.base.util.EncryptUtil;
import com.recovery.common.base.util.RedisUtil;
import com.recovery.common.base.utils.JwtUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.security.authentication.AccountExpiredException;
import org.springframework.security.authentication.DisabledException;
import org.springframework.security.authentication.LockedException;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
* @author Administrator
* @description 针对表【oauth_client_details(存储客户端的配置信息)】的数据库操作Service
* @createDate 2024-03-04 10:33:19
*/
@Slf4j
@Service
public class AuthServiceImpl implements AuthService {

    @Resource
    UserFeignClient userFeignClient;
    @Resource
    RedisUtil redisUtil;

    @Override
    public HashMap login(JwtAuthenticationRequest authenticationRequest, HttpServletRequest request) {
        HashMap map = new HashMap();
        if(null == authenticationRequest.getUsername() || "".equals(authenticationRequest.getUsername())){
            throw new BusinessException("账户不能为空");
        }
        if(null == authenticationRequest.getPassword() || "".equals(authenticationRequest.getPassword())){
            throw new BusinessException("密码不能为空");
        }

        ApiResult<UserAuthDTO> result = userFeignClient.getUserByUsername(authenticationRequest.getUsername());
        UserAuthorityDto userDetails = new UserAuthorityDto();
        if (ResultCode.SUCCESS.getCode().equals(result.getCode())) {
            UserAuthDTO user = result.getData();
            if (null != user) {
                userDetails.setUserName(user.getUserName());
                userDetails.setId(user.getUserId());
                userDetails.setPassword(user.getPassword());
            }
        }

        if (Objects.isNull(userDetails)) {
            throw new BusinessException(result.getMsg());
        }
        //加密
        if (userDetails.getUserName().equals(authenticationRequest.getUsername()) && userDetails.getPassword().equals(EncryptUtil.encrypt(authenticationRequest.getPassword()))) {
            log.info("密码校验成功！");
        } else {
            throw new BusinessException("密码错误！");
        }
        //用户信息
        map.put("userId", userDetails.toString());
        map.put("name", userDetails.getName());
        map.put("userName", userDetails.getUserName());
        map.put("password",userDetails.getPassword());
        //生成token
        String token = JwtUtils.generateToken(map);
        //token
        map.put("userInfo", userDetails);
        map.put("token", token);

        //认证通过 使用userid 生成jwt token令牌
        try {
            boolean resultRedis = redisUtil.set("userToken:" + token,userDetails , 24 * 600 * 7);
            if (!resultRedis) {
                throw new RuntimeException("网络链接失败，登录失败");
            }
        } catch (Exception e) {
            log.error(e.getMessage());
        }
       return map;
    }
}
