package com.recovery.common.base.util;



import com.alibaba.druid.pool.DruidDataSource;
import com.alibaba.fastjson.JSON;
import com.recovery.common.base.ds.DataSourceContextHolder;
import com.recovery.common.base.ds.DynamicDataSource;
import com.recovery.common.base.po.base.DBInfo;
import com.recovery.common.base.utils.RedisUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.StringRedisTemplate;

import javax.annotation.Resource;
import java.util.Map;

@Slf4j
public class HspHostUtil {

    @Resource
    private static RedisUtil redisUtil;
    //医院代码
    private static final ThreadLocal<String> HSP_HSOT = new ThreadLocal<String>();
    public static String getHspHost() {
        return HSP_HSOT.get();
    }

    public static void setHspHost(String hspHost) {
        HSP_HSOT.set(hspHost);
    }


    /**
     * 跟进域名和模块切库
     * @param host
     */
    public static void switchDB(String host, String module, StringRedisTemplate redisTemplate){
        String dataSourceKey = "dynamic-slave"+"_"+host;
        Map<Object, Object> dataSourceMap = DynamicDataSource.getInstance().getDataSourceMap();
        if (!dataSourceMap.containsKey(dataSourceKey)) {
            DBInfo dbInfo = RedisUtils.getDBInfoByHostAndModule(redisTemplate,host,module);
            if (UtilTools.isEmpty(dbInfo)){
                log.error("host > {}, module > {}",host,module);
                throw new RuntimeException("暂无查到数据库信息");
            }
            DruidDataSource dynamicDataSource = new DruidDataSource();
            dynamicDataSource.setDriverClassName("com.mysql.cj.jdbc.Driver");
            dynamicDataSource.setUrl("jdbc:mysql://" + dbInfo.getDbIp() + ":" + dbInfo.getDbPort() + "/" + dbInfo.getDbName() + "?serverTimezone=UTC&useUnicode=true&characterEncoding=utf8&allowMultiQueries=true&useSSL=false&rewriteBatchedStatements=true&serverTimezone=Asia/Shanghai");
            dynamicDataSource.setUsername(dbInfo.getDbUser());
            dynamicDataSource.setPassword(dbInfo.getDbPasswd());
                //连接配置
//            dynamicDataSource.setTestOnBorrow(Boolean.parseBoolean(EnvironmentConfig.getProperty("master.datasource.testOnBorrow")));
//            dynamicDataSource.setTestWhileIdle(Boolean.parseBoolean(EnvironmentConfig.getProperty("master.datasource.testWhileIdle")));
//            if(StringUtils.isNotBlank(EnvironmentConfig.getProperty("master.datasource.maxWait"))){
//                dynamicDataSource.setMaxWait(Integer.parseInt(EnvironmentConfig.getProperty("master.datasource.maxWait")));
//            }
//            if(StringUtils.isNotBlank(EnvironmentConfig.getProperty("master.datasource.maxActive"))){
//                dynamicDataSource.setMaxActive(Integer.parseInt(EnvironmentConfig.getProperty("master.datasource.maxActive")));
//            }
//            dynamicDataSource.setValidationQuery(EnvironmentConfig.getProperty("master.datasource.validationQuery"));
//            if(StringUtils.isNotBlank(EnvironmentConfig.getProperty("master.datasource.minIdle"))){
//                dynamicDataSource.setMinIdle(Integer.parseInt(EnvironmentConfig.getProperty("master.datasource.minIdle")));
//            }
//            if(StringUtils.isNotBlank(EnvironmentConfig.getProperty("master.datasource.minIdle"))){
//                dynamicDataSource.setInitialSize(Integer.parseInt(EnvironmentConfig.getProperty("master.datasource.minIdle")));
//            }

            //log.info("maxActive : " + EnvironmentConfig.getProperty("master.datasource.maxActive"));
            //log.info("minIdle : " + EnvironmentConfig.getProperty("master.datasource.minIdle"));
            //log.info("maxWait : " + EnvironmentConfig.getProperty("master.datasource.maxWait"));


//            dynamicDataSource.setBreakAfterAcquireFailure(true);//失败之后中断
//            dynamicDataSource.setConnectionErrorRetryAttempts(3);//失败后重试次数
//            dynamicDataSource.setMaxWait(3000);//超时时间
            dataSourceMap.put(dataSourceKey, dynamicDataSource);
            DynamicDataSource.getInstance().setTargetDataSources(dataSourceMap);
        }
        DruidDataSource druidDataSource = (DruidDataSource) dataSourceMap.get(dataSourceKey);
        //log.info("host:{},module:{},url:{}",host,module,druidDataSource.getUrl());
        DataSourceContextHolder.setDBType(dataSourceKey);
//        if(RedisUtil.getWholeDomainMap(redisTemplate).containsKey(host)){
//            //传递域名
//            HspHostUtil.setHspHost(host);
//        } else{
//            //传递域名
//            HspHostUtil.setHspHost("xxx."+host);
//        }
        //传递域名
        HspHostUtil.setHspHost(host);
    }

    /**
     * 根据db信息切库
     * @param dbInfo
     */
    public static void switchDB(DBInfo dbInfo,StringRedisTemplate redisTemplate){
        String host = dbInfo.getHospitalCode();
        String dataSourceKey = "dynamic-slave"+"_"+host;
        Map<Object, Object> dataSourceMap = DynamicDataSource.getInstance().getDataSourceMap();
        if (!dataSourceMap.containsKey(dataSourceKey)) {
            DruidDataSource dynamicDataSource = new DruidDataSource();
            dynamicDataSource.setDriverClassName("com.mysql.jdbc.Driver");
            dynamicDataSource.setUrl("jdbc:mysql://" + dbInfo.getDbIp() + ":" + dbInfo.getDbPort() + "/" + dbInfo.getDbName() + "?characterEncoding=utf-8&useSSL=false&zeroDateTimeBehavior=convertToNull&allowMultiQueries=true&serverTimezone=Asia/Shanghai");
            dynamicDataSource.setUsername(dbInfo.getDbUser());
            dynamicDataSource.setPassword(dbInfo.getDbPasswd());
//            dynamicDataSource.setBreakAfterAcquireFailure(true);//失败之后中断
//            dynamicDataSource.setConnectionErrorRetryAttempts(3);//失败后重试次数
//            dynamicDataSource.setMaxWait(3000);//超时时间
            dataSourceMap.put(dataSourceKey, dynamicDataSource);
            DynamicDataSource.getInstance().setTargetDataSources(dataSourceMap);
        }
        DataSourceContextHolder.setDBType(dataSourceKey);
//        if(RedisUtil.getWholeDomainMap(redisTemplate).containsKey(host)){
//            //传递域名
//            HspHostUtil.setHspHost(host);
//        } else{
//            //传递域名
//            HspHostUtil.setHspHost("xxx."+host);
//        }
        //传递域名
        HspHostUtil.setHspHost(host);
    }


    /**
     * 过滤全域名配置获取域名
     * @param hospitalHost
     * @param stringRedisTemplate
     * @return
     */
    public static String getWholeHost(String hospitalHost,StringRedisTemplate stringRedisTemplate){
        String domainStr = stringRedisTemplate.opsForValue().get("HOE_WHOLE_DOMAIN_REDIS_KEY");
        Map<String,String> domainMap =  JSON.parseObject(domainStr,Map.class);
        if(domainMap == null || !domainMap.containsKey(hospitalHost)){
            hospitalHost = hospitalHost.substring(hospitalHost.indexOf(".") + 1);
        }
        return hospitalHost;
    }
}
